package cs214.webapp

object Config:
  /** Which port the websocket server uses */
  val WS_PORT = 9090

  /** Which port the HTTP server uses */
  val HTTP_PORT = 8080

/** Unique identifier for an app */
type AppID = String

/** Unique identifier for a UI */
type UIID = String

/** Unique identifier for an instance of an app */
type InstanceID = String

/** Unique identifier for users */
type UserID = String

/** Encodes an object of type [[T]] to a [[ujson.Value]] */
trait Encoder[T]:
  def encode(t: T): ujson.Value

/** Decodes an object of type [[T]] from a [[ujson.Value]] */
trait Decoder[T]:
  def decode(json: ujson.Value): util.Try[T]

/** Provides a way to decode and encode an object of type [[T]] to [[Value]] */
trait WireFormat[T] extends Encoder[T] with Decoder[T]

/** HTTP endpoints */
object Endpoints:
  sealed abstract case class Api(path: String):
    val root = "/api"
    override def toString = f"$root/$path"

  object Api:
    object listApps extends Api("list-apps")
    object createInstance extends Api("create-instance")
    object instanceInfo extends Api("instance-info")

trait RegistrationProvider:
  def register(): Unit

/** Using ??? in a val breaks worksheet evaluation, so below we use TODO
  * instead. This TODO value is a hack!
  */
def TODO[A]: A = null.asInstanceOf[A]
